package com.example.student.service;

import com.example.student.model.Student;
import com.example.student.repository.StudentRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

@Service
public class StudentService {

    @Autowired
    private StudentRepository studentRepository;

    public Student createAndUpdateStudent(Student student) {
        student.setActive(true);
        return studentRepository.save(student);
    }

    public List<Student> findAllStudents() {
        return studentRepository.findAll()
                .stream()
                .filter(Student::isActive)
                .collect(Collectors.toList());
    }

    public Optional<Student> findStudentById(int id) {
        return studentRepository.findById(id).filter(Student::isActive);
    }

    public boolean deleteStudentById(int id) {
        boolean isDeleted = false;
        Optional<Student> myStudent = studentRepository.findById(id).filter(Student::isActive);
        if (myStudent.isPresent()){
            myStudent.get().setActive(false);
            isDeleted = true;
            myStudent.ifPresent(student -> studentRepository.save(student));
        }
        return isDeleted;
    }

    public List<Student> findStudentByAge(int age) {
        return studentRepository.findStudentByAge(age);
    }

    public List<Student> findStudentByHobby(String hobby) {
        return studentRepository.findStudentByHobby(hobby);
    }

    public List<Student> findStudentByTeacherId(int teacherId) {
        return studentRepository.findByTeacherId(teacherId);
    }

    public List<Student> findStudentAfterADateOfBirth(Date dateOfBirth) {
        List<Student> studentList = studentRepository.findAll();
        return studentList
                .stream()
                .filter(Student::isActive)
                .filter(x -> x.getDateOfBirth().after(dateOfBirth))
                .collect(Collectors.toList());
    }

    public List<Student> findStudentBeforeADateOfBirth(Date dateOfBirth) {
        List<Student> studentList = studentRepository.findAll();
        return studentList
                .stream()
                .filter(Student::isActive)
                .filter(x -> x.getDateOfBirth().before(dateOfBirth))
                .collect(Collectors.toList());
    }
}
