package com.example.teacher.controller;


import com.example.teacher.custom_exception.TeacherNotFoundException;
import com.example.teacher.model.Teacher;
import com.example.teacher.service.TeacherService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;
import java.util.Optional;

@RestController
@RequestMapping("/teachers")
public class TeacherController {

    @Autowired
    private TeacherService teacherService;

    // POST methods
    @PostMapping
    @ResponseStatus(HttpStatus.BAD_REQUEST)
    public ResponseEntity<Teacher> createTeacher(@Valid @RequestBody Teacher teacher) {
        Teacher myTeacher = teacherService.createAndUpdateTeacher(teacher);
        return new ResponseEntity<Teacher>(myTeacher, HttpStatus.CREATED);
    }

    // GET methods
    @GetMapping("/{id}")
    public ResponseEntity<Optional<Teacher>> findTeacherById(@PathVariable int id) throws TeacherNotFoundException {
        Optional<Teacher> myTeacher = teacherService.findTeacherById(id);
        if (myTeacher.isPresent()) {
            return new ResponseEntity<Optional<Teacher>>(myTeacher, HttpStatus.FOUND);
        }
        return new ResponseEntity<>(HttpStatus.NOT_FOUND);
    }

    @GetMapping("/all")
    public ResponseEntity<List<Teacher>> findAllTeachers() {
        List<Teacher> studentList = teacherService.findAllTeachers();
        return new ResponseEntity<List<Teacher>>(studentList, HttpStatus.OK);
    }

    @GetMapping("/name/{name}")
    public ResponseEntity<Optional<Teacher>> findTeacherByName(@PathVariable String name) {
        Optional<Teacher> myTeacher = teacherService.findTeacherByName(name);
        if (myTeacher.isPresent()) {
            return new ResponseEntity<Optional<Teacher>>(myTeacher, HttpStatus.FOUND);
        }
        return new ResponseEntity<>(HttpStatus.NOT_FOUND);
    }


    @GetMapping("/age-subject/{age}/{subject}")
    public ResponseEntity<List<Teacher>> findTeacherByAgeAndSubject(@PathVariable int age, @PathVariable String subject) {
        List<Teacher> teacherList = teacherService.findTeacherByAgeAndSubject(age, subject);
        return new ResponseEntity<List<Teacher>>(teacherList, HttpStatus.FOUND);
    }

    @GetMapping("/age/{age}")
    public ResponseEntity<List<Teacher>> findTeacherByAge(@PathVariable int age) {
        List<Teacher> teacherList = teacherService.findTeacherByAge(age);
        return new ResponseEntity<List<Teacher>>(teacherList, HttpStatus.OK);
    }

    // PUT methods
    @PutMapping("/{id}")
    public ResponseEntity<Teacher> updateTeacher(@PathVariable int id, @RequestBody Teacher teacher) {
        teacher.setId(id);
        Teacher myTeacher = teacherService.createAndUpdateTeacher(teacher);
        return new ResponseEntity<Teacher>(myTeacher, HttpStatus.ACCEPTED);
    }

    // DELETE methods
    @DeleteMapping("/delete/{id}")
    public ResponseEntity<String> deleteTeacherById(@PathVariable int id) {
        boolean isDeleted = teacherService.deleteTeacherById(id);
        if (isDeleted) {
            return new ResponseEntity<String>("Teacher deleted", HttpStatus.NO_CONTENT);
        }
        return new ResponseEntity<String>("No Teacher found", HttpStatus.NOT_FOUND);

    }

}
