package com.example.teacher.service;

import com.example.teacher.custom_exception.TeacherNotFoundException;
import com.example.teacher.model.Teacher;
import com.example.teacher.repository.TeacherRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.Collectors;

@Service
public class TeacherService {

    @Autowired
    private TeacherRepository teacherRepository;

    public Teacher createAndUpdateTeacher(Teacher teacher) {
        teacher.setActive(true);
        return teacherRepository.save(teacher);
    }

    public List<Teacher> findAllTeachers() {

        return teacherRepository.findAll()
                .stream()
                .filter(Teacher::isActive)
                .collect(Collectors.toList());
    }

    public Optional<Teacher> findTeacherById(int id) throws TeacherNotFoundException {
        return Optional.ofNullable(teacherRepository
                .findById(id)
                .filter(Teacher::isActive)
                .orElseThrow(() -> new TeacherNotFoundException("teacher with id"+ id + " found")));
    }

    public Optional<Teacher> findTeacherByName(String name) {
        return teacherRepository
                .findTeacherByName(name)
                .filter(Teacher::isActive);
    }

    public boolean deleteTeacherById(int id) {
        Teacher teacher = teacherRepository.findTeacherById(id);
        boolean isDeleted = false;
        if ((teacher != null) && (teacher.isActive())) {
            isDeleted = true;
            teacher.setActive(false);
            teacherRepository.save(teacher);
        }
        return isDeleted;
    }

    public List<Teacher> findTeacherByAgeAndSubject(int age, String subject) {
        List<Teacher> teacherList = teacherRepository.findTeacherByAgeAndSubject(age, subject);
        return teacherList
                .stream()
                .filter(Objects::nonNull)
                .filter(Teacher::isActive)
                .collect(Collectors.toList());
    }

    public List<Teacher> findTeacherByAge(int age) {
        return teacherRepository.findTeacherByAge(age)
                .stream()
                .filter(Teacher::isActive)
                .collect(Collectors.toList());
    }
}
